import tkinter as tk
from task import *

class TaskTree(tk.Frame):
	"""Il s'agit de la classe qui contient les données et l'affichage de l'arbre des tâches de l'affaire en cours"""
	
	def __init__(self, master=None):
		"""Constructeur de la classe TaskTree"""
		
		tk.Frame.__init__(self, master, bg="yellow")
		
		#On place le TaskTree dans la fenêtre principale
		self.grid(row = 2, column = 3, sticky=tk.N+tk.S+tk.W+tk.E)
		
		# $$$$$$ A changer quand il y aura les unités avant le taskContainer
		self.rowconfigure(0,weight=1)
		self.columnconfigure(0,weight=1)
		
		#Les propriétés du taskTree, conteneur principal des tâches
		self.allTasks = {} #conteneur de l'ensemble des tâches et sous-tâches de l'affaire. Les tâches quant à elles ne possèdent que l'id de leur parent/enfants
		self.selectedTasks = [] #il s'agit d'une liste que l'on va trier en fonction du level de la tâche pour faciliter la suppression
		
		#Ce qui contiendra toutes les tâches, une sorte de tâche de base qui représente l'affaire en cours
		self.taskContainer = tk.Frame(self, bg="white")
		self.taskContainer.grid(padx = 10, sticky=tk.N+tk.S+tk.W+tk.E)
		self.taskContainer.columnconfigure(0,weight=1)
		self.taskContainer.level = -1 #le level représente la profondeur d'une tâche dans l'arbre, -1 représentant cette tâche principale
		self.allTasks[-1]=self.taskContainer #-1 représente l'id de la tâche principale
		
		#les event
		self.bind_all("<Delete>", self.deleteSelectedTasks)
		self.bind_all("<BackSpace>", self.deleteSelectedTasks)
		
		#tests ----------------------------------------------------------------
		task = Task(-1, self)
		task.grid(sticky=tk.N+tk.S+tk.W+tk.E, pady=(0,5))
		
		task2 = Task(-1, self)
		task2.grid(sticky=tk.N+tk.S+tk.W+tk.E, pady=(0,5))
		
		task3 = Task(-1, self)
		task3.grid(sticky=tk.N+tk.S+tk.W+tk.E, pady=(0,5))
		
		subtask = Task(task.id, self)
		task.addSubtask(subtask)
		
		subtask2 = Task(subtask.id, self)
		subtask.addSubtask(subtask2)
		
		subtask3 = Task(task.id, self)
		task.addSubtask(subtask3)
		#----------------------------------------------------------------------
		
		
	def deleteTask(self, taskid):
		""" Fonction supprimant une tâche grâce à son id fourni en argument
			@ taskid : l'id de la tâche à supprimer """
		
		for id in self.allTasks[taskid].subtasks: #on supprime les sous-tâches
			self.deleteTask(id) 
			
		self.allTasks[taskid].destroy() #on supprime la tâche
		del self.allTasks[taskid] #et on l'enlève du dictionnaire contenant les tâches
			
			
	def selectTask(self, taskid, shifton):
		""" Selectionne la tâche d'id taskid
			@taskid : id de la tâche à selectionner/deselectionner
			@shifton : si la touche Shift est enfoncé ou non, pour la sélection multiple"""
			
		
		#on met ici à jour la liste selectedTasks pour voir si tous les id présents dedans représentent encore des tâches existantes
		toDelete = []
		for e in self.selectedTasks:
			if not e in self.allTasks:
				toDelete.append(e) #ici, la tâche d'id e n'est plus existante (plus dans allTasks)
				
		for e in toDelete: #on supprime ces id
			self.selectedTasks.remove(e)
			
		
		if not shifton: #premier cas, la touche Shift n'est pas enfoncé
			if self.selectedTasks == [taskid]: #si la tache est sélectionné et que c'est la seule, on la déselectionne
				self.allTasks[taskid].select(False)
				self.selectedTasks = []
				
			else: #s'il y a plusieurs tâches sélectionnées, on les déselectionne et on selectionne seulement taskid
				for e in self.selectedTasks:
					self.allTasks[e].select(False)
				self.selectedTasks = [taskid]
				self.allTasks[taskid].select(True)
				
		else: #la touche Shift est enfoncée
			if taskid in self.selectedTasks: #taskid est déja sélectionné, on la déselectionne mais ne touchons pas au reste
				self.allTasks[taskid].select(False)
				self.selectedTasks.remove(taskid)
			else: #elle n'est pas sélectionné, on l'ajoute aux autres, en faisant attention de garder notre liste de tâches sélectionnées triées par rapport au level
				self.allTasks[taskid].select(True)
				i = 0
				while i < len(self.selectedTasks) and self.allTasks[self.selectedTasks[i]].level > self.allTasks[taskid].level:
					i+=1
				self.selectedTasks.insert(i, taskid)
			
		
	def deleteSelectedTasks(self, event):
		""" Fonction pour supprimer les tâches sélectionnées. On appelle la fonction _delete de la tâche	
			et non directement deleteTask du taskTree pour faire en sorte que la tâche soit déréférencée chez son parent/enfants
			On simule en fait l'appui sur le bouton delete"""
		
		for e in self.selectedTasks:
			self.allTasks[e]._delete()
				
		self.selectedTasks = []
		
		
		
		
			